/*
 * SaferoomController.java
 *
 * Copyright (C) 2005-2006 Tommi Laukkanen
 * http://www.substanceofcode.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package com.substanceofcode.saferoom.control;

import com.substanceofcode.saferoom.data.*;
import com.substanceofcode.utils.Settings;
import java.io.IOException;
import javax.microedition.midlet.MIDlet;
import javax.microedition.rms.RecordStoreException;

/**
 *
 * @author Tommi Laukkanen
 */
public class SaferoomController {

    /** Properties */
    private static SaferoomController m_controller = null;
    private Settings m_settings = null;
    private PasswordGroupCollection m_groups = null;
    private MIDlet m_midlet = null;
    private String m_password = "";
    
    /** Creates a new instance of SaferoomController */
    private SaferoomController(MIDlet midlet)
            throws IOException, RecordStoreException {
	m_settings = Settings.getInstance( midlet );
        m_midlet = midlet;
        m_groups = new PasswordGroupCollection();
    }
    
    /** Get singleton instance */
    public static SaferoomController getInstance(MIDlet midlet)
            throws IOException, RecordStoreException {
	if( m_controller == null ) {
	    m_controller = new SaferoomController( midlet );
        }
	return m_controller;
    }
    
    /** 
     * Open saferoom repository with the given password.
     * 
     * @return true if password was correct and repository opened succesfully.
     */                  
    public boolean openDatabase(String password) {
        m_password = password;
        try {
            SaferoomRepository repository;
            repository = SaferoomRepository.getInstance();
            m_groups = repository.load(m_midlet, password, false);
            if(m_groups==null) {
                return false;
            }
        } catch(Exception e) {
            System.err.println("Error while loading repository: " + e.toString());
            return false;
        }
        return true;
    }
    
    /** Get password groups */
    public String[] getGroups() {
        String[] groups = m_groups.getNames();
	return groups;	
    }
    
    /** Get password group by name */
    public PasswordGroup getGroup(String groupName) {
        PasswordGroup group = m_groups.getByName(groupName);
        return group;
    }
    
    /** Add new group */
    public void addGroup(PasswordGroup group) {
	m_groups.add( group );
    }
    
    /** Delete group */
    public void deleteGroup(PasswordGroup group) {
        m_groups.remove(group);
    }
    
    /** Close saferoom */
    public void closeDatabase() {
        SaferoomRepository repository;
        repository = SaferoomRepository.getInstance();
        repository.save( m_midlet, m_groups, m_password );
    }

    /** Add password to password group */
    public PasswordGroupCollection addPassword(PasswordItem password, PasswordGroup group) {
        PasswordGroup oldGroup = m_groups.getByName(group.getName());
        PasswordItemCollection passwords = oldGroup.getPasswords();
        passwords.add(password);
        oldGroup.setPasswords( passwords );
        m_groups.add(oldGroup);
        return m_groups;
    }

    /** Get specific password from password group */
    public PasswordItem getPassword(PasswordGroup group, String passwordLocation) {
        PasswordItemCollection passwords = group.getPasswords();
        PasswordItem password = passwords.getItemByLocation(passwordLocation);
        return password;
    }

    /** Remove specific password from password group */
    public PasswordGroupCollection removePassword(PasswordGroup group, String location) {
        PasswordGroup oldGroup = m_groups.getByName( group.getName());
        PasswordItemCollection passwords = oldGroup.getPasswords();
        PasswordItem deletedPassword = passwords.getItemByLocation( location );
        passwords.remove( deletedPassword );
        oldGroup.setPasswords( passwords );
        m_groups.add( oldGroup );
        return m_groups;
    }

    /** Set the current repository password */
    public void setPassword(String password) {
        m_password = password;
    }

    
}
